@@ -50,15 +50,19 @@ aSPUwpermC <- function(Y, X, cov = NULL, model=c("gaussian", "binomial"), pow=c(
         U<-as.vector( t(Xg) %*% r)
 
         ##cov of the score stats:
-        CovS<- mean(Y)*(1-mean(Y))*(t(Xgb) %*% Xgb)
-
+        if( model == "binomial" ) {
+            CovS <- mean(Y)*(1-mean(Y))*(t(Xgb) %*% Xgb)
+        } else {
+            CovS <- var(Y)*(t(Xgb) %*% Xgb)
+        }
+        
     } else {
         ## with nuisance parameters:
         tdat1<-data.frame(trait=Y, cov)
         fit1<-glm(trait~.,family=model,data=tdat1)
         pis<-fitted.values(fit1)
         XUs<-matrix(0, nrow=n, ncol=k)
-	r <- Y - pis
+        r <- Y - pis
         for(i in 1:k){
             tdat2<-data.frame(X1=X[,i], cov)
             fit2<-glm(X1~.,data=tdat2)
@@ -67,9 +71,12 @@ aSPUwpermC <- function(Y, X, cov = NULL, model=c("gaussian", "binomial"), pow=c(
         }
         U <- t(XUs) %*% (Y - pis)
 
-        CovS<-matrix(0, nrow=k, ncol=k)
-        for(i in 1:n)
-            CovS<-CovS + XUs[i,] %*% t(XUs[i,])
+        if( model == "binomial" ) {
+            CovS <- mean(pis*(1-pis))*(t(XUs) %*% XUs)
+        } else {
+            CovS <- var(r)*(t(XUs) %*% XUs)
+        }        
+
     }
 
 
